@@ -1,4 +1,4 @@
-// Copyright 2016-2017 Francesco Biscani (bluescarni@gmail.com)
+// Copyright 2016-2018 Francesco Biscani (bluescarni@gmail.com)
 //
 // This file is part of the mp++ library.
 //
@@ -1006,6 +1006,9 @@ public:
     __float128 m_value;
 };
 
+// Double check that real128 is a standard layout class.
+static_assert(std::is_standard_layout<real128>::value, "real128 is not a standard layout class.");
+
 /** @defgroup real128_conversion real128_conversion
  *  @{
  */
@@ -1424,7 +1427,7 @@ inline real128 dispatch_pow(const T &x, const real128 &y)
  */
 #if defined(MPPP_HAVE_CONCEPTS)
 template <typename T>
-inline real128 pow(const T &x, const Real128OpTypes<T> &y)
+inline real128 pow(const Real128OpTypes<T> &x, const T &y)
 #else
 template <typename T, typename U, real128_op_types_enabler<T, U> = 0>
 inline real128 pow(const T &x, const U &y)
@@ -1558,7 +1561,7 @@ constexpr real128 dispatch_add(const T &x, const real128 &y)
  */
 #if defined(MPPP_HAVE_CONCEPTS)
 template <typename T>
-constexpr real128 operator+(const T &x, const Real128CppOpTypes<T> &y)
+constexpr real128 operator+(const Real128CppOpTypes<T> &x, const T &y)
 #else
 template <typename T, typename U, real128_cpp_op_types_enabler<T, U> = 0>
 constexpr real128 operator+(const T &x, const U &y)
@@ -1595,7 +1598,7 @@ inline real128 dispatch_add(const T &x, const real128 &y)
  */
 #if defined(MPPP_HAVE_CONCEPTS)
 template <typename T>
-inline real128 operator+(const T &x, const Real128MpppOpTypes<T> &y)
+inline real128 operator+(const Real128MpppOpTypes<T> &x, const T &y)
 #else
 template <typename T, typename U, real128_mppp_op_types_enabler<T, U> = 0>
 inline real128 operator+(const T &x, const U &y)
@@ -1642,7 +1645,7 @@ inline MPPP_CONSTEXPR_14 void dispatch_in_place_add(T &x, const real128 &y)
  */
 #if defined(MPPP_HAVE_CONCEPTS)
 template <typename T>
-inline MPPP_CONSTEXPR_14 T &operator+=(T &x, const Real128CppOpTypes<T> &y)
+inline MPPP_CONSTEXPR_14 auto &operator+=(Real128CppOpTypes<T> &x, const T &y)
 #else
 template <typename T, typename U, real128_cpp_op_types_enabler<T, U> = 0>
 inline MPPP_CONSTEXPR_14 T &operator+=(T &x, const U &y)
@@ -1680,7 +1683,7 @@ inline void dispatch_in_place_add(T &x, const real128 &y)
  */
 #if defined(MPPP_HAVE_CONCEPTS)
 template <typename T>
-inline T &operator+=(T &x, const Real128MpppOpTypes<T> &y)
+inline auto &operator+=(Real128MpppOpTypes<T> &x, const T &y)
 #else
 template <typename T, typename U, real128_mppp_op_types_enabler<T, U> = 0>
 inline T &operator+=(T &x, const U &y)
@@ -1772,7 +1775,7 @@ constexpr real128 dispatch_sub(const T &x, const real128 &y)
  */
 #if defined(MPPP_HAVE_CONCEPTS)
 template <typename T>
-constexpr real128 operator-(const T &x, const Real128CppOpTypes<T> &y)
+constexpr real128 operator-(const Real128CppOpTypes<T> &x, const T &y)
 #else
 template <typename T, typename U, real128_cpp_op_types_enabler<T, U> = 0>
 constexpr real128 operator-(const T &x, const U &y)
@@ -1809,7 +1812,7 @@ inline real128 dispatch_sub(const T &x, const real128 &y)
  */
 #if defined(MPPP_HAVE_CONCEPTS)
 template <typename T>
-inline real128 operator-(const T &x, const Real128MpppOpTypes<T> &y)
+inline real128 operator-(const Real128MpppOpTypes<T> &x, const T &y)
 #else
 template <typename T, typename U, real128_mppp_op_types_enabler<T, U> = 0>
 inline real128 operator-(const T &x, const U &y)
@@ -1854,7 +1857,7 @@ inline MPPP_CONSTEXPR_14 void dispatch_in_place_sub(T &x, const real128 &y)
  */
 #if defined(MPPP_HAVE_CONCEPTS)
 template <typename T>
-inline MPPP_CONSTEXPR_14 T &operator-=(T &x, const Real128CppOpTypes<T> &y)
+inline MPPP_CONSTEXPR_14 auto &operator-=(Real128CppOpTypes<T> &x, const T &y)
 #else
 template <typename T, typename U, real128_cpp_op_types_enabler<T, U> = 0>
 inline MPPP_CONSTEXPR_14 T &operator-=(T &x, const U &y)
@@ -1892,7 +1895,7 @@ inline void dispatch_in_place_sub(T &x, const real128 &y)
  */
 #if defined(MPPP_HAVE_CONCEPTS)
 template <typename T>
-inline T &operator-=(T &x, const Real128MpppOpTypes<T> &y)
+inline auto &operator-=(Real128MpppOpTypes<T> &x, const T &y)
 #else
 template <typename T, typename U, real128_mppp_op_types_enabler<T, U> = 0>
 inline T &operator-=(T &x, const U &y)
@@ -1973,7 +1976,7 @@ constexpr real128 dispatch_mul(const T &x, const real128 &y)
  */
 #if defined(MPPP_HAVE_CONCEPTS)
 template <typename T>
-constexpr real128 operator*(const T &x, const Real128CppOpTypes<T> &y)
+constexpr real128 operator*(const Real128CppOpTypes<T> &x, const T &y)
 #else
 template <typename T, typename U, real128_cpp_op_types_enabler<T, U> = 0>
 constexpr real128 operator*(const T &x, const U &y)
@@ -2010,7 +2013,7 @@ inline real128 dispatch_mul(const T &x, const real128 &y)
  */
 #if defined(MPPP_HAVE_CONCEPTS)
 template <typename T>
-inline real128 operator*(const T &x, const Real128MpppOpTypes<T> &y)
+inline real128 operator*(const Real128MpppOpTypes<T> &x, const T &y)
 #else
 template <typename T, typename U, real128_mppp_op_types_enabler<T, U> = 0>
 inline real128 operator*(const T &x, const U &y)
@@ -2055,7 +2058,7 @@ inline MPPP_CONSTEXPR_14 void dispatch_in_place_mul(T &x, const real128 &y)
  */
 #if defined(MPPP_HAVE_CONCEPTS)
 template <typename T>
-inline MPPP_CONSTEXPR_14 T &operator*=(T &x, const Real128CppOpTypes<T> &y)
+inline MPPP_CONSTEXPR_14 auto &operator*=(Real128CppOpTypes<T> &x, const T &y)
 #else
 template <typename T, typename U, real128_cpp_op_types_enabler<T, U> = 0>
 inline MPPP_CONSTEXPR_14 T &operator*=(T &x, const U &y)
@@ -2093,7 +2096,7 @@ inline void dispatch_in_place_mul(T &x, const real128 &y)
  */
 #if defined(MPPP_HAVE_CONCEPTS)
 template <typename T>
-inline T &operator*=(T &x, const Real128MpppOpTypes<T> &y)
+inline auto &operator*=(Real128MpppOpTypes<T> &x, const T &y)
 #else
 template <typename T, typename U, real128_mppp_op_types_enabler<T, U> = 0>
 inline T &operator*=(T &x, const U &y)
@@ -2133,7 +2136,7 @@ constexpr real128 dispatch_div(const T &x, const real128 &y)
  */
 #if defined(MPPP_HAVE_CONCEPTS)
 template <typename T>
-constexpr real128 operator/(const T &x, const Real128CppOpTypes<T> &y)
+constexpr real128 operator/(const Real128CppOpTypes<T> &x, const T &y)
 #else
 template <typename T, typename U, real128_cpp_op_types_enabler<T, U> = 0>
 constexpr real128 operator/(const T &x, const U &y)
@@ -2170,7 +2173,7 @@ inline real128 dispatch_div(const T &x, const real128 &y)
  */
 #if defined(MPPP_HAVE_CONCEPTS)
 template <typename T>
-inline real128 operator/(const T &x, const Real128MpppOpTypes<T> &y)
+inline real128 operator/(const Real128MpppOpTypes<T> &x, const T &y)
 #else
 template <typename T, typename U, real128_mppp_op_types_enabler<T, U> = 0>
 inline real128 operator/(const T &x, const U &y)
@@ -2215,7 +2218,7 @@ inline MPPP_CONSTEXPR_14 void dispatch_in_place_div(T &x, const real128 &y)
  */
 #if defined(MPPP_HAVE_CONCEPTS)
 template <typename T>
-inline MPPP_CONSTEXPR_14 T &operator/=(T &x, const Real128CppOpTypes<T> &y)
+inline MPPP_CONSTEXPR_14 auto &operator/=(Real128CppOpTypes<T> &x, const T &y)
 #else
 template <typename T, typename U, real128_cpp_op_types_enabler<T, U> = 0>
 inline MPPP_CONSTEXPR_14 T &operator/=(T &x, const U &y)
@@ -2253,7 +2256,7 @@ inline void dispatch_in_place_div(T &x, const real128 &y)
  */
 #if defined(MPPP_HAVE_CONCEPTS)
 template <typename T>
-inline T &operator/=(T &x, const Real128MpppOpTypes<T> &y)
+inline auto &operator/=(Real128MpppOpTypes<T> &x, const T &y)
 #else
 template <typename T, typename U, real128_mppp_op_types_enabler<T, U> = 0>
 inline T &operator/=(T &x, const U &y)
@@ -2302,7 +2305,7 @@ constexpr bool dispatch_eq(const T &x, const real128 &y)
  */
 #if defined(MPPP_HAVE_CONCEPTS)
 template <typename T>
-constexpr bool operator==(const T &x, const Real128CppOpTypes<T> &y)
+constexpr bool operator==(const Real128CppOpTypes<T> &x, const T &y)
 #else
 template <typename T, typename U, real128_cpp_op_types_enabler<T, U> = 0>
 constexpr bool operator==(const T &x, const U &y)
@@ -2349,7 +2352,7 @@ inline bool dispatch_eq(const T &x, const real128 &y)
  */
 #if defined(MPPP_HAVE_CONCEPTS)
 template <typename T>
-inline bool operator==(const T &x, const Real128MpppOpTypes<T> &y)
+inline bool operator==(const Real128MpppOpTypes<T> &x, const T &y)
 #else
 template <typename T, typename U, real128_mppp_op_types_enabler<T, U> = 0>
 inline bool operator==(const T &x, const U &y)
@@ -2376,7 +2379,7 @@ inline bool operator==(const T &x, const U &y)
  */
 #if defined(MPPP_HAVE_CONCEPTS)
 template <typename T>
-constexpr bool operator!=(const T &x, const Real128CppOpTypes<T> &y)
+constexpr bool operator!=(const Real128CppOpTypes<T> &x, const T &y)
 #else
 template <typename T, typename U, real128_cpp_op_types_enabler<T, U> = 0>
 constexpr bool operator!=(const T &x, const U &y)
@@ -2407,7 +2410,7 @@ constexpr bool operator!=(const T &x, const U &y)
  */
 #if defined(MPPP_HAVE_CONCEPTS)
 template <typename T>
-inline bool operator!=(const T &x, const Real128MpppOpTypes<T> &y)
+inline bool operator!=(const Real128MpppOpTypes<T> &x, const T &y)
 #else
 template <typename T, typename U, real128_mppp_op_types_enabler<T, U> = 0>
 inline bool operator!=(const T &x, const U &y)
@@ -2456,7 +2459,7 @@ constexpr bool dispatch_lt(const T &x, const real128 &y)
  */
 #if defined(MPPP_HAVE_CONCEPTS)
 template <typename T>
-constexpr bool operator<(const T &x, const Real128CppOpTypes<T> &y)
+constexpr bool operator<(const Real128CppOpTypes<T> &x, const T &y)
 #else
 template <typename T, typename U, real128_cpp_op_types_enabler<T, U> = 0>
 constexpr bool operator<(const T &x, const U &y)
@@ -2503,7 +2506,7 @@ inline bool dispatch_lt(const T &x, const real128 &y)
  */
 #if defined(MPPP_HAVE_CONCEPTS)
 template <typename T>
-inline bool operator<(const T &x, const Real128MpppOpTypes<T> &y)
+inline bool operator<(const Real128MpppOpTypes<T> &x, const T &y)
 #else
 template <typename T, typename U, real128_mppp_op_types_enabler<T, U> = 0>
 inline bool operator<(const T &x, const U &y)
@@ -2550,7 +2553,7 @@ constexpr bool dispatch_lte(const T &x, const real128 &y)
  */
 #if defined(MPPP_HAVE_CONCEPTS)
 template <typename T>
-constexpr bool operator<=(const T &x, const Real128CppOpTypes<T> &y)
+constexpr bool operator<=(const Real128CppOpTypes<T> &x, const T &y)
 #else
 template <typename T, typename U, real128_cpp_op_types_enabler<T, U> = 0>
 constexpr bool operator<=(const T &x, const U &y)
@@ -2595,7 +2598,7 @@ inline bool dispatch_lte(const T &x, const real128 &y)
  */
 #if defined(MPPP_HAVE_CONCEPTS)
 template <typename T>
-inline bool operator<=(const T &x, const Real128MpppOpTypes<T> &y)
+inline bool operator<=(const Real128MpppOpTypes<T> &x, const T &y)
 #else
 template <typename T, typename U, real128_mppp_op_types_enabler<T, U> = 0>
 inline bool operator<=(const T &x, const U &y)
@@ -2644,7 +2647,7 @@ constexpr bool dispatch_gt(const T &x, const real128 &y)
  */
 #if defined(MPPP_HAVE_CONCEPTS)
 template <typename T>
-constexpr bool operator>(const T &x, const Real128CppOpTypes<T> &y)
+constexpr bool operator>(const Real128CppOpTypes<T> &x, const T &y)
 #else
 template <typename T, typename U, real128_cpp_op_types_enabler<T, U> = 0>
 constexpr bool operator>(const T &x, const U &y)
@@ -2691,7 +2694,7 @@ inline bool dispatch_gt(const T &x, const real128 &y)
  */
 #if defined(MPPP_HAVE_CONCEPTS)
 template <typename T>
-inline bool operator>(const T &x, const Real128MpppOpTypes<T> &y)
+inline bool operator>(const Real128MpppOpTypes<T> &x, const T &y)
 #else
 template <typename T, typename U, real128_mppp_op_types_enabler<T, U> = 0>
 inline bool operator>(const T &x, const U &y)
@@ -2738,7 +2741,7 @@ constexpr bool dispatch_gte(const T &x, const real128 &y)
  */
 #if defined(MPPP_HAVE_CONCEPTS)
 template <typename T>
-constexpr bool operator>=(const T &x, const Real128CppOpTypes<T> &y)
+constexpr bool operator>=(const Real128CppOpTypes<T> &x, const T &y)
 #else
 template <typename T, typename U, real128_cpp_op_types_enabler<T, U> = 0>
 constexpr bool operator>=(const T &x, const U &y)
@@ -2783,7 +2786,7 @@ inline bool dispatch_gte(const T &x, const real128 &y)
  */
 #if defined(MPPP_HAVE_CONCEPTS)
 template <typename T>
-inline bool operator>=(const T &x, const Real128MpppOpTypes<T> &y)
+inline bool operator>=(const Real128MpppOpTypes<T> &x, const T &y)
 #else
 template <typename T, typename U, real128_mppp_op_types_enabler<T, U> = 0>
 inline bool operator>=(const T &x, const U &y)
